@@ -12,41 +12,42 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Representation for the MongoDB internal MinKey type.
-"""
+"""Representation for the MongoDB internal MinKey type."""
+from typing import Any
 
 
-class MinKey(object):
+class MinKey:
     """MongoDB internal MinKey type."""
+
     __slots__ = ()
 
     _type_marker = 255
 
-    def __getstate__(self):
+    def __getstate__(self) -> Any:
         return {}
 
-    def __setstate__(self, state):
+    def __setstate__(self, state: Any) -> None:
         pass
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         return isinstance(other, MinKey)
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash(self._type_marker)
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
-    def __le__(self, dummy):
+    def __le__(self, dummy: Any) -> bool:
         return True
 
-    def __lt__(self, other):
+    def __lt__(self, other: Any) -> bool:
         return not isinstance(other, MinKey)
 
-    def __ge__(self, other):
+    def __ge__(self, other: Any) -> bool:
         return isinstance(other, MinKey)
 
-    def __gt__(self, dummy):
+    def __gt__(self, dummy: Any) -> bool:
         return False
 
     def __repr__(self):
