@@ -13,8 +13,9 @@
 # limitations under the License.
 
 """Exceptions raised by PyMongo."""
+from typing import Any, Iterable, List, Mapping, Optional, Sequence, Tuple, Union
 
-from bson.errors import *
+from bson.errors import InvalidDocument
 
 try:
     # CPython 3.7+
@@ -23,18 +24,20 @@ except ImportError:
     try:
         from ssl import CertificateError as _CertificateError
     except ImportError:
-        class _CertificateError(ValueError):
+
+        class _CertificateError(ValueError):  # type: ignore
             pass
 
 
 class PyMongoError(Exception):
     """Base class for all PyMongo exceptions."""
-    def __init__(self, message='', error_labels=None):
-        super(PyMongoError, self).__init__(message)
+
+    def __init__(self, message: str = "", error_labels: Optional[Iterable[str]] = None) -> None:
+        super().__init__(message)
         self._message = message
         self._error_labels = set(error_labels or [])
 
-    def has_error_label(self, label):
+    def has_error_label(self, label: str) -> bool:
         """Return True if this error contains the given label.
 
         .. versionadded:: 3.7
@@ -49,6 +52,14 @@ class PyMongoError(Exception):
         """Remove the given label from this error."""
         self._error_labels.discard(label)
 
+    @property
+    def timeout(self) -> bool:
+        """True if this error was caused by a timeout.
+
+        .. versionadded:: 4.2
+        """
+        return False
+
 
 class ProtocolError(PyMongoError):
     """Raised for failures related to the wire protocol."""
@@ -58,6 +69,19 @@ class ConnectionFailure(PyMongoError):
     """Raised when a connection to the database cannot be made or is lost."""
 
 
+class WaitQueueTimeoutError(ConnectionFailure):
+    """Raised when an operation times out waiting to checkout a connection from the pool.
+
+    Subclass of :exc:`~pymongo.errors.ConnectionFailure`.
+
+    .. versionadded:: 4.2
+    """
+
+    @property
+    def timeout(self) -> bool:
+        return True
+
+
 class AutoReconnect(ConnectionFailure):
     """Raised when a connection to the database is lost and an attempt to
     auto-reconnect will be made.
@@ -70,11 +94,18 @@ class AutoReconnect(ConnectionFailure):
 
     Subclass of :exc:`~pymongo.errors.ConnectionFailure`.
     """
-    def __init__(self, message='', errors=None):
+
+    errors: Union[Mapping[str, Any], Sequence]
+    details: Union[Mapping[str, Any], Sequence]
+
+    def __init__(
+        self, message: str = "", errors: Optional[Union[Mapping[str, Any], Sequence]] = None
+    ) -> None:
         error_labels = None
-        if errors is not None and isinstance(errors, dict):
-            error_labels = errors.get('errorLabels')
-        super(AutoReconnect, self).__init__(message, error_labels)
+        if errors is not None:
+            if isinstance(errors, dict):
+                error_labels = errors.get("errorLabels")
+        super().__init__(message, error_labels)
         self.errors = self.details = errors or []
 
 
@@ -87,10 +118,14 @@ class NetworkTimeout(AutoReconnect):
     Subclass of :exc:`~pymongo.errors.AutoReconnect`.
     """
 
+    @property
+    def timeout(self) -> bool:
+        return True
+
 
 def _format_detailed_error(message, details):
     if details is not None:
-        message = "%s, full error: %s" % (message, details)
+        message = f"{message}, full error: {details}"
     return message
 
 
@@ -109,9 +144,11 @@ class NotPrimaryError(AutoReconnect):
 
     .. versionadded:: 3.12
     """
-    def __init__(self, message='', errors=None):
-        super(NotPrimaryError, self).__init__(
-            _format_detailed_error(message, errors), errors=errors)
+
+    def __init__(
+        self, message: str = "", errors: Optional[Union[Mapping[str, Any], List]] = None
+    ) -> None:
+        super().__init__(_format_detailed_error(message, errors), errors=errors)
 
 
 class ServerSelectionTimeoutError(AutoReconnect):
@@ -126,10 +163,13 @@ class ServerSelectionTimeoutError(AutoReconnect):
     Preference that the replica set cannot satisfy.
     """
 
+    @property
+    def timeout(self) -> bool:
+        return True
+
 
 class ConfigurationError(PyMongoError):
-    """Raised when something is incorrectly configured.
-    """
+    """Raised when something is incorrectly configured."""
 
 
 class OperationFailure(PyMongoError):
@@ -139,12 +179,17 @@ class OperationFailure(PyMongoError):
        The :attr:`details` attribute.
     """
 
-    def __init__(self, error, code=None, details=None, max_wire_version=None):
+    def __init__(
+        self,
+        error: str,
+        code: Optional[int] = None,
+        details: Optional[Mapping[str, Any]] = None,
+        max_wire_version: Optional[int] = None,
+    ) -> None:
         error_labels = None
         if details is not None:
-            error_labels = details.get('errorLabels')
-        super(OperationFailure, self).__init__(
-            _format_detailed_error(error, details), error_labels=error_labels)
+            error_labels = details.get("errorLabels")
+        super().__init__(_format_detailed_error(error, details), error_labels=error_labels)
         self.__code = code
         self.__details = details
         self.__max_wire_version = max_wire_version
@@ -154,13 +199,12 @@ class OperationFailure(PyMongoError):
         return self.__max_wire_version
 
     @property
-    def code(self):
-        """The error code returned by the server, if any.
-        """
+    def code(self) -> Optional[int]:
+        """The error code returned by the server, if any."""
         return self.__code
 
     @property
-    def details(self):
+    def details(self) -> Optional[Mapping[str, Any]]:
         """The complete error document returned by the server.
 
         Depending on the error that occurred, the error document
@@ -171,6 +215,9 @@ class OperationFailure(PyMongoError):
         """
         return self.__details
 
+    @property
+    def timeout(self) -> bool:
+        return self.__code in (50,)
 
 
 class CursorNotFound(OperationFailure):
@@ -190,6 +237,10 @@ class ExecutionTimeout(OperationFailure):
     .. versionadded:: 2.7
     """
 
+    @property
+    def timeout(self) -> bool:
+        return True
+
 
 class WriteConcernError(OperationFailure):
     """Base exception type for errors raised due to write concern.
@@ -215,23 +266,47 @@ class WTimeoutError(WriteConcernError):
     .. versionadded:: 2.7
     """
 
+    @property
+    def timeout(self) -> bool:
+        return True
+
 
 class DuplicateKeyError(WriteError):
     """Raised when an insert or update fails due to a duplicate key error."""
 
 
+def _wtimeout_error(error: Any) -> bool:
+    """Return True if this writeConcernError doc is a caused by a timeout."""
+    return error.get("code") == 50 or ("errInfo" in error and error["errInfo"].get("wtimeout"))
+
+
 class BulkWriteError(OperationFailure):
     """Exception class for bulk write errors.
 
     .. versionadded:: 2.7
     """
-    def __init__(self, results):
-        super(BulkWriteError, self).__init__(
-            "batch op errors occurred", 65, results)
 
-    def __reduce__(self):
+    details: Mapping[str, Any]
+
+    def __init__(self, results: Mapping[str, Any]) -> None:
+        super().__init__("batch op errors occurred", 65, results)
+
+    def __reduce__(self) -> Tuple[Any, Any]:
         return self.__class__, (self.details,)
 
+    @property
+    def timeout(self) -> bool:
+        # Check the last writeConcernError and last writeError to determine if this
+        # BulkWriteError was caused by a timeout.
+        wces = self.details.get("writeConcernErrors", [])
+        if wces and _wtimeout_error(wces[-1]):
+            return True
+
+        werrs = self.details.get("writeErrors", [])
+        if werrs and werrs[-1].get("code") == 50:
+            return True
+        return False
+
 
 class InvalidOperation(PyMongoError):
     """Raised when a client attempts to perform an invalid operation."""
@@ -250,9 +325,7 @@ class InvalidURI(ConfigurationError):
 
 
 class DocumentTooLarge(InvalidDocument):
-    """Raised when an encoded document is too large for the connected server.
-    """
-    pass
+    """Raised when an encoded document is too large for the connected server."""
 
 
 class EncryptionError(PyMongoError):
@@ -264,17 +337,43 @@ class EncryptionError(PyMongoError):
     .. versionadded:: 3.9
     """
 
-    def __init__(self, cause):
-        super(EncryptionError, self).__init__(str(cause))
+    def __init__(self, cause: Exception) -> None:
+        super().__init__(str(cause))
         self.__cause = cause
 
     @property
-    def cause(self):
+    def cause(self) -> Exception:
         """The exception that caused this encryption or decryption error."""
         return self.__cause
 
+    @property
+    def timeout(self) -> bool:
+        if isinstance(self.__cause, PyMongoError):
+            return self.__cause.timeout
+        return False
+
+
+class EncryptedCollectionError(EncryptionError):
+    """Raised when creating a collection with encrypted_fields fails.
+
+    .. versionadded:: 4.4
+    """
+
+    def __init__(self, cause: Exception, encrypted_fields: Mapping[str, Any]) -> None:
+        super().__init__(cause)
+        self.__encrypted_fields = encrypted_fields
+
+    @property
+    def encrypted_fields(self) -> Mapping[str, Any]:
+        """The encrypted_fields document that allows inferring which data keys are *known* to be created.
+
+        Note that the returned document is not guaranteed to contain information about *all* of the data keys that
+        were created, for example in the case of an indefinite error like a timeout. Use the `cause` property to
+        determine whether a definite or indefinite error caused this error, and only rely on the accuracy of the
+        encrypted_fields if the error is definite.
+        """
+        return self.__encrypted_fields
+
 
 class _OperationCancelled(AutoReconnect):
-    """Internal error raised when a socket operation is cancelled.
-    """
-    pass
+    """Internal error raised when a socket operation is cancelled."""
