@@ -15,14 +15,15 @@
 """Class to monitor a MongoDB server on a background thread."""
 
 import atexit
-import threading
 import time
 import weakref
 from typing import Any, Mapping, cast
 
 from pymongo import common, periodic_executor
+from pymongo._csot import MovingMinimum
 from pymongo.errors import NotPrimaryError, OperationFailure, _OperationCancelled
 from pymongo.hello import Hello
+from pymongo.lock import _create_lock
 from pymongo.periodic_executor import _shutdown_executors
 from pymongo.read_preferences import MovingAverage
 from pymongo.server_description import ServerDescription
@@ -36,11 +37,11 @@ def _sanitize(error):
     error.__cause__ = None
 
 
-class MonitorBase(object):
+class MonitorBase:
     def __init__(self, topology, name, interval, min_interval):
         """Base class to do periodic work on a background thread.
 
-        The the background thread is signaled to stop when the Topology or
+        The background thread is signaled to stop when the Topology or
         this instance is freed.
         """
         # We strongly reference the executor and it weakly references us via
@@ -107,7 +108,7 @@ class Monitor(MonitorBase):
         The Topology is weakly referenced. The Pool must be exclusive to this
         Monitor.
         """
-        super(Monitor, self).__init__(
+        super().__init__(
             topology,
             "pymongo_server_monitor_thread",
             topology_settings.heartbeat_frequency,
@@ -250,7 +251,8 @@ class Monitor(MonitorBase):
             if not response.awaitable:
                 self._rtt_monitor.add_sample(round_trip_time)
 
-            sd = ServerDescription(address, response, self._rtt_monitor.average())
+            avg_rtt, min_rtt = self._rtt_monitor.get()
+            sd = ServerDescription(address, response, avg_rtt, min_round_trip_time=min_rtt)
             if self._publish:
                 self._listeners.publish_server_heartbeat_succeeded(
                     address, round_trip_time, response, response.awaitable
@@ -288,7 +290,7 @@ class SrvMonitor(MonitorBase):
 
         The Topology is weakly referenced.
         """
-        super(SrvMonitor, self).__init__(
+        super().__init__(
             topology,
             "pymongo_srv_polling_thread",
             common.MIN_SRV_RESCAN_INTERVAL,
@@ -341,7 +343,7 @@ class _RttMonitor(MonitorBase):
 
         The Topology is weakly referenced.
         """
-        super(_RttMonitor, self).__init__(
+        super().__init__(
             topology,
             "pymongo_server_rtt_thread",
             topology_settings.heartbeat_frequency,
@@ -350,7 +352,8 @@ class _RttMonitor(MonitorBase):
 
         self._pool = pool
         self._moving_average = MovingAverage()
-        self._lock = threading.Lock()
+        self._moving_min = MovingMinimum()
+        self._lock = _create_lock()
 
     def close(self):
         self.gc_safe_close()
@@ -362,20 +365,22 @@ class _RttMonitor(MonitorBase):
         """Add a RTT sample."""
         with self._lock:
             self._moving_average.add_sample(sample)
+            self._moving_min.add_sample(sample)
 
-    def average(self):
-        """Get the calculated average, or None if no samples yet."""
+    def get(self):
+        """Get the calculated average, or None if no samples yet and the min."""
         with self._lock:
-            return self._moving_average.get()
+            return self._moving_average.get(), self._moving_min.get()
 
     def reset(self):
         """Reset the average RTT."""
         with self._lock:
-            return self._moving_average.reset()
+            self._moving_average.reset()
+            self._moving_min.reset()
 
     def _run(self):
         try:
-            # NOTE: This thread is only run when when using the streaming
+            # NOTE: This thread is only run when using the streaming
             # heartbeat protocol (MongoDB 4.4+).
             # XXX: Skip check if the server is unknown?
             rtt = self._ping()
@@ -430,10 +435,10 @@ def _shutdown_monitors():
 def _shutdown_resources():
     # _shutdown_monitors/_shutdown_executors may already be GC'd at shutdown.
     shutdown = _shutdown_monitors
-    if shutdown:
+    if shutdown:  # type:ignore[truthy-function]
         shutdown()
     shutdown = _shutdown_executors
-    if shutdown:
+    if shutdown:  # type:ignore[truthy-function]
         shutdown()
 
 
