@@ -125,6 +125,9 @@ Connection monitoring and pooling events are also available. For example::
         def pool_created(self, event):
             logging.info("[pool {0.address}] pool created".format(event))
 
+        def pool_ready(self, event):
+            logging.info("[pool {0.address}] pool is ready".format(event))
+
         def pool_cleared(self, event):
             logging.info("[pool {0.address}] pool cleared".format(event))
 
@@ -180,6 +183,8 @@ will not add that listener to existing client instances.
   handler first.
 """
 
+from __future__ import annotations
+
 import datetime
 from collections import abc, namedtuple
 from typing import TYPE_CHECKING, Any, Dict, Optional
@@ -208,7 +213,7 @@ _Listeners = namedtuple(
 _LISTENERS = _Listeners([], [], [], [], [])
 
 
-class _EventListener(object):
+class _EventListener:
     """Abstract base class for all event listeners."""
 
 
@@ -483,14 +488,14 @@ def _to_micros(dur):
 def _validate_event_listeners(option, listeners):
     """Validate event listeners"""
     if not isinstance(listeners, abc.Sequence):
-        raise TypeError("%s must be a list or tuple" % (option,))
+        raise TypeError(f"{option} must be a list or tuple")
     for listener in listeners:
         if not isinstance(listener, _EventListener):
             raise TypeError(
-                "Listeners for %s must be either a "
+                "Listeners for {} must be either a "
                 "CommandListener, ServerHeartbeatListener, "
                 "ServerListener, TopologyListener, or "
-                "ConnectionPoolListener." % (option,)
+                "ConnectionPoolListener.".format(option)
             )
     return listeners
 
@@ -505,10 +510,10 @@ def register(listener: _EventListener) -> None:
     """
     if not isinstance(listener, _EventListener):
         raise TypeError(
-            "Listeners for %s must be either a "
+            "Listeners for {} must be either a "
             "CommandListener, ServerHeartbeatListener, "
             "ServerListener, TopologyListener, or "
-            "ConnectionPoolListener." % (listener,)
+            "ConnectionPoolListener.".format(listener)
         )
     if isinstance(listener, CommandListener):
         _LISTENERS.command_listeners.append(listener)
@@ -525,19 +530,17 @@ def register(listener: _EventListener) -> None:
 # Note - to avoid bugs from forgetting which if these is all lowercase and
 # which are camelCase, and at the same time avoid having to add a test for
 # every command, use all lowercase here and test against command_name.lower().
-_SENSITIVE_COMMANDS = set(
-    [
-        "authenticate",
-        "saslstart",
-        "saslcontinue",
-        "getnonce",
-        "createuser",
-        "updateuser",
-        "copydbgetnonce",
-        "copydbsaslstart",
-        "copydb",
-    ]
-)
+_SENSITIVE_COMMANDS: set = {
+    "authenticate",
+    "saslstart",
+    "saslcontinue",
+    "getnonce",
+    "createuser",
+    "updateuser",
+    "copydbgetnonce",
+    "copydbsaslstart",
+    "copydb",
+}
 
 
 # The "hello" command is also deemed sensitive when attempting speculative
@@ -551,7 +554,7 @@ def _is_speculative_authenticate(command_name, doc):
     return False
 
 
-class _CommandEvent(object):
+class _CommandEvent:
     """Base class for command events."""
 
     __slots__ = ("__cmd_name", "__rqst_id", "__conn_id", "__op_id", "__service_id")
@@ -624,10 +627,10 @@ class CommandStartedEvent(_CommandEvent):
         service_id: Optional[ObjectId] = None,
     ) -> None:
         if not command:
-            raise ValueError("%r is not a valid command" % (command,))
+            raise ValueError(f"{command!r} is not a valid command")
         # Command name must be first key.
         command_name = next(iter(command))
-        super(CommandStartedEvent, self).__init__(
+        super().__init__(
             command_name, request_id, connection_id, operation_id, service_id=service_id
         )
         cmd_name = command_name.lower()
@@ -648,7 +651,7 @@ class CommandStartedEvent(_CommandEvent):
         return self.__db
 
     def __repr__(self):
-        return ("<%s %s db: %r, command: %r, operation_id: %s, service_id: %s>") % (
+        return ("<{} {} db: {!r}, command: {!r}, operation_id: {}, service_id: {}>").format(
             self.__class__.__name__,
             self.connection_id,
             self.database_name,
@@ -684,7 +687,7 @@ class CommandSucceededEvent(_CommandEvent):
         operation_id: Optional[int],
         service_id: Optional[ObjectId] = None,
     ) -> None:
-        super(CommandSucceededEvent, self).__init__(
+        super().__init__(
             command_name, request_id, connection_id, operation_id, service_id=service_id
         )
         self.__duration_micros = _to_micros(duration)
@@ -705,7 +708,9 @@ class CommandSucceededEvent(_CommandEvent):
         return self.__reply
 
     def __repr__(self):
-        return ("<%s %s command: %r, operation_id: %s, duration_micros: %s, service_id: %s>") % (
+        return (
+            "<{} {} command: {!r}, operation_id: {}, duration_micros: {}, service_id: {}>"
+        ).format(
             self.__class__.__name__,
             self.connection_id,
             self.command_name,
@@ -741,7 +746,7 @@ class CommandFailedEvent(_CommandEvent):
         operation_id: Optional[int],
         service_id: Optional[ObjectId] = None,
     ) -> None:
-        super(CommandFailedEvent, self).__init__(
+        super().__init__(
             command_name, request_id, connection_id, operation_id, service_id=service_id
         )
         self.__duration_micros = _to_micros(duration)
@@ -759,9 +764,9 @@ class CommandFailedEvent(_CommandEvent):
 
     def __repr__(self):
         return (
-            "<%s %s command: %r, operation_id: %s, duration_micros: %s, "
-            "failure: %r, service_id: %s>"
-        ) % (
+            "<{} {} command: {!r}, operation_id: {}, duration_micros: {}, "
+            "failure: {!r}, service_id: {}>"
+        ).format(
             self.__class__.__name__,
             self.connection_id,
             self.command_name,
@@ -772,7 +777,7 @@ class CommandFailedEvent(_CommandEvent):
         )
 
 
-class _PoolEvent(object):
+class _PoolEvent:
     """Base class for pool events."""
 
     __slots__ = ("__address",)
@@ -788,7 +793,7 @@ class _PoolEvent(object):
         return self.__address
 
     def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self.__address)
+        return f"{self.__class__.__name__}({self.__address!r})"
 
 
 class PoolCreatedEvent(_PoolEvent):
@@ -804,7 +809,7 @@ class PoolCreatedEvent(_PoolEvent):
     __slots__ = ("__options",)
 
     def __init__(self, address: _Address, options: Dict[str, Any]) -> None:
-        super(PoolCreatedEvent, self).__init__(address)
+        super().__init__(address)
         self.__options = options
 
     @property
@@ -813,7 +818,7 @@ class PoolCreatedEvent(_PoolEvent):
         return self.__options
 
     def __repr__(self):
-        return "%s(%r, %r)" % (self.__class__.__name__, self.address, self.__options)
+        return f"{self.__class__.__name__}({self.address!r}, {self.__options!r})"
 
 
 class PoolReadyEvent(_PoolEvent):
@@ -843,7 +848,7 @@ class PoolClearedEvent(_PoolEvent):
     __slots__ = ("__service_id",)
 
     def __init__(self, address: _Address, service_id: Optional[ObjectId] = None) -> None:
-        super(PoolClearedEvent, self).__init__(address)
+        super().__init__(address)
         self.__service_id = service_id
 
     @property
@@ -857,7 +862,7 @@ class PoolClearedEvent(_PoolEvent):
         return self.__service_id
 
     def __repr__(self):
-        return "%s(%r, %r)" % (self.__class__.__name__, self.address, self.__service_id)
+        return f"{self.__class__.__name__}({self.address!r}, {self.__service_id!r})"
 
 
 class PoolClosedEvent(_PoolEvent):
@@ -873,7 +878,7 @@ class PoolClosedEvent(_PoolEvent):
     __slots__ = ()
 
 
-class ConnectionClosedReason(object):
+class ConnectionClosedReason:
     """An enum that defines values for `reason` on a
     :class:`ConnectionClosedEvent`.
 
@@ -894,7 +899,7 @@ class ConnectionClosedReason(object):
     """The pool was closed, making the connection no longer valid."""
 
 
-class ConnectionCheckOutFailedReason(object):
+class ConnectionCheckOutFailedReason:
     """An enum that defines values for `reason` on a
     :class:`ConnectionCheckOutFailedEvent`.
 
@@ -913,14 +918,13 @@ class ConnectionCheckOutFailedReason(object):
     """
 
 
-class _ConnectionEvent(object):
-    """Private base class for some connection events."""
+class _ConnectionEvent:
+    """Private base class for connection events."""
 
-    __slots__ = ("__address", "__connection_id")
+    __slots__ = ("__address",)
 
-    def __init__(self, address: _Address, connection_id: int) -> None:
+    def __init__(self, address: _Address) -> None:
         self.__address = address
-        self.__connection_id = connection_id
 
     @property
     def address(self) -> _Address:
@@ -929,16 +933,29 @@ class _ConnectionEvent(object):
         """
         return self.__address
 
+    def __repr__(self):
+        return f"{self.__class__.__name__}({self.__address!r})"
+
+
+class _ConnectionIdEvent(_ConnectionEvent):
+    """Private base class for connection events with an id."""
+
+    __slots__ = ("__connection_id",)
+
+    def __init__(self, address: _Address, connection_id: int) -> None:
+        super().__init__(address)
+        self.__connection_id = connection_id
+
     @property
     def connection_id(self) -> int:
         """The ID of the Connection."""
         return self.__connection_id
 
     def __repr__(self):
-        return "%s(%r, %r)" % (self.__class__.__name__, self.__address, self.__connection_id)
+        return f"{self.__class__.__name__}({self.address!r}, {self.__connection_id!r})"
 
 
-class ConnectionCreatedEvent(_ConnectionEvent):
+class ConnectionCreatedEvent(_ConnectionIdEvent):
     """Published when a Connection Pool creates a Connection object.
 
     NOTE: This connection is not ready for use until the
@@ -955,7 +972,7 @@ class ConnectionCreatedEvent(_ConnectionEvent):
     __slots__ = ()
 
 
-class ConnectionReadyEvent(_ConnectionEvent):
+class ConnectionReadyEvent(_ConnectionIdEvent):
     """Published when a Connection has finished its setup, and is ready to use.
 
     :Parameters:
@@ -969,7 +986,7 @@ class ConnectionReadyEvent(_ConnectionEvent):
     __slots__ = ()
 
 
-class ConnectionClosedEvent(_ConnectionEvent):
+class ConnectionClosedEvent(_ConnectionIdEvent):
     """Published when a Connection is closed.
 
     :Parameters:
@@ -984,7 +1001,7 @@ class ConnectionClosedEvent(_ConnectionEvent):
     __slots__ = ("__reason",)
 
     def __init__(self, address, connection_id, reason):
-        super(ConnectionClosedEvent, self).__init__(address, connection_id)
+        super().__init__(address, connection_id)
         self.__reason = reason
 
     @property
@@ -997,7 +1014,7 @@ class ConnectionClosedEvent(_ConnectionEvent):
         return self.__reason
 
     def __repr__(self):
-        return "%s(%r, %r, %r)" % (
+        return "{}({!r}, {!r}, {!r})".format(
             self.__class__.__name__,
             self.address,
             self.connection_id,
@@ -1005,7 +1022,7 @@ class ConnectionClosedEvent(_ConnectionEvent):
         )
 
 
-class ConnectionCheckOutStartedEvent(object):
+class ConnectionCheckOutStartedEvent(_ConnectionEvent):
     """Published when the driver starts attempting to check out a connection.
 
     :Parameters:
@@ -1015,23 +1032,10 @@ class ConnectionCheckOutStartedEvent(object):
     .. versionadded:: 3.9
     """
 
-    __slots__ = ("__address",)
-
-    def __init__(self, address):
-        self.__address = address
-
-    @property
-    def address(self):
-        """The address (host, port) pair of the server this connection is
-        attempting to connect to.
-        """
-        return self.__address
-
-    def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self.__address)
+    __slots__ = ()
 
 
-class ConnectionCheckOutFailedEvent(object):
+class ConnectionCheckOutFailedEvent(_ConnectionEvent):
     """Published when the driver's attempt to check out a connection fails.
 
     :Parameters:
@@ -1042,19 +1046,12 @@ class ConnectionCheckOutFailedEvent(object):
     .. versionadded:: 3.9
     """
 
-    __slots__ = ("__address", "__reason")
+    __slots__ = ("__reason",)
 
     def __init__(self, address: _Address, reason: str) -> None:
-        self.__address = address
+        super().__init__(address)
         self.__reason = reason
 
-    @property
-    def address(self) -> _Address:
-        """The address (host, port) pair of the server this connection is
-        attempting to connect to.
-        """
-        return self.__address
-
     @property
     def reason(self) -> str:
         """A reason explaining why connection check out failed.
@@ -1065,10 +1062,10 @@ class ConnectionCheckOutFailedEvent(object):
         return self.__reason
 
     def __repr__(self):
-        return "%s(%r, %r)" % (self.__class__.__name__, self.__address, self.__reason)
+        return f"{self.__class__.__name__}({self.address!r}, {self.__reason!r})"
 
 
-class ConnectionCheckedOutEvent(_ConnectionEvent):
+class ConnectionCheckedOutEvent(_ConnectionIdEvent):
     """Published when the driver successfully checks out a Connection.
 
     :Parameters:
@@ -1082,7 +1079,7 @@ class ConnectionCheckedOutEvent(_ConnectionEvent):
     __slots__ = ()
 
 
-class ConnectionCheckedInEvent(_ConnectionEvent):
+class ConnectionCheckedInEvent(_ConnectionIdEvent):
     """Published when the driver checks in a Connection into the Pool.
 
     :Parameters:
@@ -1096,7 +1093,7 @@ class ConnectionCheckedInEvent(_ConnectionEvent):
     __slots__ = ()
 
 
-class _ServerEvent(object):
+class _ServerEvent:
     """Base class for server events."""
 
     __slots__ = ("__server_address", "__topology_id")
@@ -1116,7 +1113,7 @@ class _ServerEvent(object):
         return self.__topology_id
 
     def __repr__(self):
-        return "<%s %s topology_id: %s>" % (
+        return "<{} {} topology_id: {}>".format(
             self.__class__.__name__,
             self.server_address,
             self.topology_id,
@@ -1133,28 +1130,30 @@ class ServerDescriptionChangedEvent(_ServerEvent):
 
     def __init__(
         self,
-        previous_description: "ServerDescription",
-        new_description: "ServerDescription",
-        *args: Any
+        previous_description: ServerDescription,
+        new_description: ServerDescription,
+        *args: Any,
     ) -> None:
-        super(ServerDescriptionChangedEvent, self).__init__(*args)
+        super().__init__(*args)
         self.__previous_description = previous_description
         self.__new_description = new_description
 
     @property
-    def previous_description(self) -> "ServerDescription":
+    def previous_description(self) -> ServerDescription:
         """The previous
-        :class:`~pymongo.server_description.ServerDescription`."""
+        :class:`~pymongo.server_description.ServerDescription`.
+        """
         return self.__previous_description
 
     @property
-    def new_description(self) -> "ServerDescription":
+    def new_description(self) -> ServerDescription:
         """The new
-        :class:`~pymongo.server_description.ServerDescription`."""
+        :class:`~pymongo.server_description.ServerDescription`.
+        """
         return self.__new_description
 
     def __repr__(self):
-        return "<%s %s changed from: %s, to: %s>" % (
+        return "<{} {} changed from: {}, to: {}>".format(
             self.__class__.__name__,
             self.server_address,
             self.previous_description,
@@ -1180,7 +1179,7 @@ class ServerClosedEvent(_ServerEvent):
     __slots__ = ()
 
 
-class TopologyEvent(object):
+class TopologyEvent:
     """Base class for topology description events."""
 
     __slots__ = "__topology_id"
@@ -1194,7 +1193,7 @@ class TopologyEvent(object):
         return self.__topology_id
 
     def __repr__(self):
-        return "<%s topology_id: %s>" % (self.__class__.__name__, self.topology_id)
+        return f"<{self.__class__.__name__} topology_id: {self.topology_id}>"
 
 
 class TopologyDescriptionChangedEvent(TopologyEvent):
@@ -1207,28 +1206,30 @@ class TopologyDescriptionChangedEvent(TopologyEvent):
 
     def __init__(
         self,
-        previous_description: "TopologyDescription",
-        new_description: "TopologyDescription",
-        *args: Any
+        previous_description: TopologyDescription,
+        new_description: TopologyDescription,
+        *args: Any,
     ) -> None:
-        super(TopologyDescriptionChangedEvent, self).__init__(*args)
+        super().__init__(*args)
         self.__previous_description = previous_description
         self.__new_description = new_description
 
     @property
-    def previous_description(self) -> "TopologyDescription":
+    def previous_description(self) -> TopologyDescription:
         """The previous
-        :class:`~pymongo.topology_description.TopologyDescription`."""
+        :class:`~pymongo.topology_description.TopologyDescription`.
+        """
         return self.__previous_description
 
     @property
-    def new_description(self) -> "TopologyDescription":
+    def new_description(self) -> TopologyDescription:
         """The new
-        :class:`~pymongo.topology_description.TopologyDescription`."""
+        :class:`~pymongo.topology_description.TopologyDescription`.
+        """
         return self.__new_description
 
     def __repr__(self):
-        return "<%s topology_id: %s changed from: %s, to: %s>" % (
+        return "<{} topology_id: {} changed from: {}, to: {}>".format(
             self.__class__.__name__,
             self.topology_id,
             self.previous_description,
@@ -1254,7 +1255,7 @@ class TopologyClosedEvent(TopologyEvent):
     __slots__ = ()
 
 
-class _ServerHeartbeatEvent(object):
+class _ServerHeartbeatEvent:
     """Base class for server heartbeat events."""
 
     __slots__ = "__connection_id"
@@ -1265,11 +1266,12 @@ class _ServerHeartbeatEvent(object):
     @property
     def connection_id(self) -> _Address:
         """The address (host, port) of the server this heartbeat was sent
-        to."""
+        to.
+        """
         return self.__connection_id
 
     def __repr__(self):
-        return "<%s %s>" % (self.__class__.__name__, self.connection_id)
+        return f"<{self.__class__.__name__} {self.connection_id}>"
 
 
 class ServerHeartbeatStartedEvent(_ServerHeartbeatEvent):
@@ -1292,7 +1294,7 @@ class ServerHeartbeatSucceededEvent(_ServerHeartbeatEvent):
     def __init__(
         self, duration: float, reply: Hello, connection_id: _Address, awaited: bool = False
     ) -> None:
-        super(ServerHeartbeatSucceededEvent, self).__init__(connection_id)
+        super().__init__(connection_id)
         self.__duration = duration
         self.__reply = reply
         self.__awaited = awaited
@@ -1318,7 +1320,7 @@ class ServerHeartbeatSucceededEvent(_ServerHeartbeatEvent):
         return self.__awaited
 
     def __repr__(self):
-        return "<%s %s duration: %s, awaited: %s, reply: %s>" % (
+        return "<{} {} duration: {}, awaited: {}, reply: {}>".format(
             self.__class__.__name__,
             self.connection_id,
             self.duration,
@@ -1339,7 +1341,7 @@ class ServerHeartbeatFailedEvent(_ServerHeartbeatEvent):
     def __init__(
         self, duration: float, reply: Exception, connection_id: _Address, awaited: bool = False
     ) -> None:
-        super(ServerHeartbeatFailedEvent, self).__init__(connection_id)
+        super().__init__(connection_id)
         self.__duration = duration
         self.__reply = reply
         self.__awaited = awaited
@@ -1365,7 +1367,7 @@ class ServerHeartbeatFailedEvent(_ServerHeartbeatEvent):
         return self.__awaited
 
     def __repr__(self):
-        return "<%s %s duration: %s, awaited: %s, reply: %r>" % (
+        return "<{} {} duration: {}, awaited: {}, reply: {!r}>".format(
             self.__class__.__name__,
             self.connection_id,
             self.duration,
@@ -1374,7 +1376,7 @@ class ServerHeartbeatFailedEvent(_ServerHeartbeatEvent):
         )
 
 
-class _EventListeners(object):
+class _EventListeners:
     """Configure event listeners for a client instance.
 
     Any event listeners registered globally are included by default.
